using System;
using System.IO;
using System.Collections;
using org.bouncycastle.asn1;
using org.bouncycastle.asn1.util;
using org.bouncycastle.asn1.x509;
using org.bouncycastle.util.encoders;
using org.bouncycastle.util.test;

namespace test.asn1
{
	public class X509NameTest: Test
	{
		public String getName()
		{
			return "X509Name";
		}
	
		public TestResult perform()
		{
			Hashtable                   attrs = new Hashtable();

			attrs[X509Name.C]  = "AU";
			attrs[X509Name.O] = "The Legion of the Bouncy Castle";
			attrs[X509Name.L] = "Melbourne";
			attrs[X509Name.ST] = "Victoria";
			attrs[X509Name.E] = "feedback-crypto@bouncycastle.org";

			X509Name    name1 = new X509Name(attrs);

			if (!name1.Equals(name1))
			{
				return new SimpleTestResult(false, getName() + ": Failed same object test");
			}

			X509Name    name2 = new X509Name(attrs);

			if (!name1.Equals(name2))
			{
				return new SimpleTestResult(false, getName() + ": Failed same name test");
			}

			ArrayList  ord1 = new ArrayList();

			ord1.Add(X509Name.C);
			ord1.Add(X509Name.O);
			ord1.Add(X509Name.L);
			ord1.Add(X509Name.ST);
			ord1.Add(X509Name.E);

			ArrayList  ord2 = new ArrayList();

			ord2.Add(X509Name.E);
			ord2.Add(X509Name.ST);
			ord2.Add(X509Name.L);
			ord2.Add(X509Name.O);
			ord2.Add(X509Name.C);

			name1 = new X509Name(ord1, attrs);
			name2 = new X509Name(ord2, attrs);

			if (!name1.Equals(name2))
			{
				return new SimpleTestResult(false, getName() + ": Failed reverse name test");
			}

			ord2 = new ArrayList();

			ord2.Add(X509Name.ST);
			ord2.Add(X509Name.ST);
			ord2.Add(X509Name.L);
			ord2.Add(X509Name.O);
			ord2.Add(X509Name.C);

			name1 = new X509Name(ord1, attrs);
			name2 = new X509Name(ord2, attrs);

			if (name1.Equals(name2))
			{
				return new SimpleTestResult(false, getName() + ": Failed different name test");
			}

			ord2 = new ArrayList();

			ord2.Add(X509Name.ST);
			ord2.Add(X509Name.L);
			ord2.Add(X509Name.O);
			ord2.Add(X509Name.C);

			name1 = new X509Name(ord1, attrs);
			name2 = new X509Name(ord2, attrs);

			if (name1.Equals(name2))
			{
				return new SimpleTestResult(false, getName() + ": Failed subset name test");
			}

			return new SimpleTestResult(true, getName() + ": Okay");
		}

		[STAThread]
		static void Main(string[] args)
		{
			Test    test = new X509NameTest();
			TestResult  result = test.perform();
			Console.WriteLine(result.ToString());
		}

	}
}
