using System;
using org.bouncycastle.util.encoders;
using org.bouncycastle.util.test;


namespace test.encoders
{
	public class EncoderTest : Test 
	{
		public static readonly bool DEBUG = true;

		/*
		 *
		 *  INFRASTRUCTURE
		 *
		 */
		[STAThread]
		static void Main(string[] args)
		{
			EncoderTest _test = new EncoderTest();
			Console.WriteLine(_test.testBase64().ToString());
			Console.WriteLine(_test.testHex().ToString());
		}

		public static void log(Exception _ex) 
		{
			if(DEBUG) 
			{
				Console.WriteLine(_ex.StackTrace);
			}
		}

		public static void log(String _msg) 
		{
			if(DEBUG) 
			{
				Console.WriteLine(_msg);
			}
		}

		public String getName()
		{
			return "Encoder";
		}
	
		/*
		 *
		 *  TESTS
		 *
		 */

		public TestResult perform()
		{
			return testBase64();
		}
	
	
		public TestResult testBase64() 
		{
			try 
			{
				Random _r = new Random();
			
				byte[] _orig1024 = new byte[1024];
				_r.NextBytes(_orig1024);
			
				byte[] _orig2048 = new byte[2048];
				_r.NextBytes(_orig2048);
			
				byte[] _orig4096 = new byte[4096];
				_r.NextBytes(_orig4096);
			
				byte[] _orig8192 = new byte[8192];
				_r.NextBytes(_orig8192);
			
				byte[] _enc1024 = Base64.encode(_orig1024);
				byte[] _enc2048 = Base64.encode(_orig2048);
				byte[] _enc4096 = Base64.encode(_orig4096);
				byte[] _enc8192 = Base64.encode(_orig8192);
			
				byte[] _dec1024 = Base64.decode(_enc1024);
				byte[] _dec2048 = Base64.decode(_enc2048);
				byte[] _dec4096 = Base64.decode(_enc4096);
				byte[] _dec8192 = Base64.decode(_enc8192);
			

				if(!equals((byte[]) _orig1024, (byte[])_dec1024)) 
				{
					return new SimpleTestResult(false, "Failed Base64 test");
				}
			
				if(!equals(_orig2048, _dec2048)) 
				{
					return new SimpleTestResult(false, "Failed Base64 test");
				}
			
				if(!equals(_orig4096, _dec4096)) 
				{
					return new SimpleTestResult(false, "Failed Base64 test");
				}
			
				if(!equals(_orig8192, _dec8192)) 
				{
					return new SimpleTestResult(false, "Failed Base64 test");
				}
			
			
			
				byte[] _orig1025 = new byte[1025];
				_r.NextBytes(_orig1025);
			
				byte[] _orig2049 = new byte[2049];
				_r.NextBytes(_orig2049);
			
				byte[] _orig4097 = new byte[4097];
				_r.NextBytes(_orig4097);
			
				byte[] _orig8193 = new byte[8193];
				_r.NextBytes(_orig8193);
			
				byte[] _enc1025 = Base64.encode(_orig1025);
				byte[] _enc2049 = Base64.encode(_orig2049);
				byte[] _enc4097 = Base64.encode(_orig4097);
				byte[] _enc8193 = Base64.encode(_orig8193);
			
				byte[] _dec1025 = Base64.decode(_enc1025);
				byte[] _dec2049 = Base64.decode(_enc2049);
				byte[] _dec4097 = Base64.decode(_enc4097);
				byte[] _dec8193 = Base64.decode(_enc8193);
			
				if(!equals(_orig1025, _dec1025)) 
				{
					return new SimpleTestResult(false, "Failed Base64 test");
				}
			
				if(!equals(_orig2049, _dec2049)) 
				{
					return new SimpleTestResult(false, "Failed Base64 test");
				}
			
				if(!equals(_orig4097, _dec4097)) 
				{
					return new SimpleTestResult(false, "Failed Base64 test");
				}
			
				if(!equals(_orig8193, _dec8193)) 
				{
					return new SimpleTestResult(false, "Failed Base64 test");
				}
			
				return new SimpleTestResult(true, "Passed Base64 test");
			}
			catch(Exception ex) 
			{
				log(ex);
				return new SimpleTestResult(false, "Failed Base64 test");
			}
		}


		public TestResult testHex() 
		{
			try 
			{
				Random _r = new Random();
			
				byte[] _orig1024 = new byte[1024];
				_r.NextBytes(_orig1024);
			
				byte[] _orig2048 = new byte[2048];
				_r.NextBytes(_orig2048);
			
				byte[] _orig4096 = new byte[4096];
				_r.NextBytes(_orig4096);
			
				byte[] _orig8192 = new byte[8192];
				_r.NextBytes(_orig8192);
			
				byte[] _enc1024 = Hex.encode(_orig1024);
				byte[] _enc2048 = Hex.encode(_orig2048);
				byte[] _enc4096 = Hex.encode(_orig4096);
				byte[] _enc8192 = Hex.encode(_orig8192);
			
				byte[] _dec1024 = Hex.decode(_enc1024);
				byte[] _dec2048 = Hex.decode(_enc2048);
				byte[] _dec4096 = Hex.decode(_enc4096);
				byte[] _dec8192 = Hex.decode(_enc8192);
			
				if(!equals(_orig1024, _dec1024)) 
				{
					return new SimpleTestResult(false, "Failed Hex test");
				}
			
				if(!equals(_orig2048, _dec2048)) 
				{
					return new SimpleTestResult(false, "Failed Hex test");
				}
			
				if(!equals(_orig4096, _dec4096)) 
				{
					return new SimpleTestResult(false, "Failed Hex test");
				}
			
				if(!equals(_orig8192, _dec8192)) 
				{
					return new SimpleTestResult(false, "Failed Hex test");
				}
			
			
				byte[] _orig1025 = new byte[1025];
				_r.NextBytes(_orig1025);
			
				byte[] _orig2049 = new byte[2049];
				_r.NextBytes(_orig2049);
			
				byte[] _orig4097 = new byte[4097];
				_r.NextBytes(_orig4097);
			
				byte[] _orig8193 = new byte[8193];
				_r.NextBytes(_orig8193);
			
				byte[] _enc1025 = Hex.encode(_orig1025);
				byte[] _enc2049 = Hex.encode(_orig2049);
				byte[] _enc4097 = Hex.encode(_orig4097);
				byte[] _enc8193 = Hex.encode(_orig8193);
			
				byte[] _dec1025 = Hex.decode(_enc1025);
				byte[] _dec2049 = Hex.decode(_enc2049);
				byte[] _dec4097 = Hex.decode(_enc4097);
				byte[] _dec8193 = Hex.decode(_enc8193);
			
				if(!equals(_orig1025, _dec1025)) 
				{
					return new SimpleTestResult(false, "Failed Hex test");
				}
			
				if(!equals(_orig2049, _dec2049)) 
				{
					return new SimpleTestResult(false, "Failed Hex test");
				}
			
				if(!equals(_orig4097, _dec4097)) 
				{
					return new SimpleTestResult(false, "Failed Hex test");
				}
			
				if(!equals(_orig8193, _dec8193)) 
				{
					return new SimpleTestResult(false, "Failed Hex test");
				}
			
				return new SimpleTestResult(true, "Passed Hex test");
			}
			catch(Exception ex) 
			{
				log(ex);
				return new SimpleTestResult(false, "Failed Hex test");
			}
		}
		bool equals(byte[] a1, byte[] a2)
		{
			if (a1.Length != a2.Length) return false;
			for (int i = 0; i < a1.Length; i++)
			{
				if (a1[i] != a2[i]) return false;
			}
			return true; 
		}
	}

}
