
using System;
//using System.Collections;
//using org.bouncycastle.io;
using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.x509
{
	/**
	* IssuingDistributionPoint ::= SEQUENCE {
	*      distributionPoint          [0] DistributionPointName OPTIONAL,
	*      onlyContainsUserCerts      [1] BOOLEAN DEFAULT FALSE,
	*      onlyContainsCACerts        [2] BOOLEAN DEFAULT FALSE,
	*      onlySomeReasons            [3] ReasonFlags OPTIONAL,
	*      indirectCRL                [4] BOOLEAN DEFAULT FALSE,
	*      onlyContainsAttributeCerts [5] BOOLEAN DEFAULT FALSE }
	*/
	public class IssuingDistributionPoint: DEREncodable
	{
		private bool         m_onlyContainsUserCerts;
		private bool         m_onlyContainsCACerts;
		private bool         m_indirectCRL;
		private bool         m_onlyContainsAttributeCerts;

		private ASN1Sequence    seq;

		public static IssuingDistributionPoint getInstance(
			ASN1TaggedObject obj,
			bool          flagExplicit)
		{
			return getInstance(ASN1Sequence.getInstance(obj, flagExplicit));
		}

		public static IssuingDistributionPoint getInstance(
			Object  obj)
		{
			if ((obj == null) || (typeof(IssuingDistributionPoint).IsInstanceOfType(obj)))
			{
				return (IssuingDistributionPoint)obj;
			}		
			else if (typeof(ASN1Sequence).IsInstanceOfType(obj))
			{
				return new IssuingDistributionPoint((ASN1Sequence)obj);
			}

			throw new ArgumentException("unknown object in factory");
		}

		/**
		* Constructor from ASN1Sequence
		*/
		public IssuingDistributionPoint(
			ASN1Sequence  seq)
		{
			this.seq = seq;

			for (int i = 0; i != seq.size(); i++)
			{
				ASN1TaggedObject  o = (ASN1TaggedObject)seq.getObjectAt(i);

				switch (o.getTagNo())
				{
				case 0:
					break;
				case 1:
					m_onlyContainsUserCerts = DERBoolean.getInstance(o, false).isTrue();
					break;
				case 2:
					m_onlyContainsCACerts = DERBoolean.getInstance(o, false).isTrue();
					break;
				case 3:
					break;
				case 4:
					m_indirectCRL = DERBoolean.getInstance(o, false).isTrue();
					break;
				case 5:
					m_onlyContainsAttributeCerts = DERBoolean.getInstance(o, false).isTrue();
					break;
				default:
					throw new ArgumentException("unknown tag in IssuingDistributionPoint");
				}
			}
		}

		public bool onlyContainsUserCerts()
		{
			return m_onlyContainsUserCerts;
		}

		public bool onlyContainsCACerts()
		{
			return m_onlyContainsCACerts;
		}

		public bool isIndirectCRL()
		{
			return m_indirectCRL;
		}

		public bool onlyContainsAttributeCerts()
		{
			return m_onlyContainsAttributeCerts;
		}

		public DERObject getDERObject()
		{
			return seq;
		}
	}

}