using System;
//using System.Collections;
//using org.bouncycastle.io;
using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.x509
{
	public class ObjectDigestInfo: DEREncodable
	{
		internal DEREnumerated       digestedObjectType = null;
		internal DERObjectIdentifier otherObjectTypeID = null; 
		internal AlgorithmIdentifier digestAlgorithm = null;
		internal DERBitString        objectDigest = null;

		public DEREnumerated getDigestedObjectType()
		{
			return digestedObjectType;
		}

		public DERObjectIdentifier getOtherObjectTypeID()
		{
			return otherObjectTypeID;
		}

		public AlgorithmIdentifier getDigestAlgorithm()
		{
			return digestAlgorithm;
		}

		public DERBitString getObjectDigest()
		{
			return objectDigest;
		}

		/**
		 * <pre>
		 *  ObjectDigestInfo ::= SEQUENCE {
		 *       digestedObjectType  ENUMERATED {
		 *               publicKey            (0),
		 *               publicKeyCert        (1),
		 *               otherObjectTypes     (2) },
		 *                       -- otherObjectTypes MUST NOT
		 *                       -- be used in this profile
		 *       otherObjectTypeID   OBJECT IDENTIFIER OPTIONAL,
		 *       digestAlgorithm     AlgorithmIdentifier,
		 *       objectDigest        BIT STRING
		 *  }
		 * </pre>
		 */
		public DERObject getDERObject()
		{
			ASN1EncodableVector  v = new ASN1EncodableVector();

			v.add(digestedObjectType);

			if (otherObjectTypeID != null)
			{
				v.add(otherObjectTypeID);
			}

			v.add(digestAlgorithm);
			v.add(objectDigest);

			return new DERSequence(v);
		}
	}

}