using System;
using System.Text;
using org.bouncycastle.util.test;
using org.bouncycastle.util.encoders;
using org.bouncycastle.crypto;

namespace org.bouncycastle.util.test
{
	/**
	 * a basic test that takes a stream cipher, key parameter, and an input
	 * and output string.
	 */
	public class StreamCipherVectorTest: Test
	{
		int                 id;
		StreamCipher        cipher;
		CipherParameters    param;
		byte[]              input;
		byte[]              output;

		public StreamCipherVectorTest(
			int                 id,
			StreamCipher        cipher,
			CipherParameters    param,
			String              input,
			String              output)
		{
			this.id = id;
			this.cipher = cipher;
			this.param = param;
			this.input = Hex.decode(input);
			this.output = Hex.decode(output);
		}

		public String getName()
		{
			return cipher.getAlgorithmName() + " Vector Test " + id;
		}

		public TestResult perform()
		{
			cipher.init(true, param);

			byte[]  outBytes = new byte[input.Length];

			cipher.processBytes(input, 0, input.Length, outBytes, 0);

			if (!isEqualArray(outBytes, output))
										  {
											  return new SimpleTestResult(false,
												  getName() + ": failed - " + "expected " + ByteArray2String(Hex.encode(output)) + " got " + ByteArray2String(Hex.encode(outBytes)));
										  }

			cipher.init(false, param);

			cipher.processBytes(output, 0, output.Length, outBytes, 0);

			if (!isEqualArray(input, outBytes))
			{
				return new SimpleTestResult(false, getName() + ": failed reversal");
			}

			return new SimpleTestResult(true, getName() + ": OKAY");
		}

		private bool isEqualArray(
			byte[]  a,
			byte[]  b)
		{
			if (a.Length != b.Length)
			{
				return false;
			}

			for (int i = 0; i != a.Length; i++)
			{
				if (a[i] != b[i])
				{
					return false;
				}
			}

			return true;
		}
		protected String ByteArray2String(byte[] bs)
		{
			StringBuilder s = new StringBuilder();
			for (int i = 0; i < bs.Length; i++) 
			{
				s.Append(Convert.ToChar(bs[i]));
			}
			return s.ToString();
		}
	}

}
